package com.aodianyun.service.api;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.security.InvalidKeyException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import com.sun.org.apache.xerces.internal.impl.dv.util.Base64;

public class TisService {
	public final static int FORMATERROR = 103;
	public final static int PARAMERROR = 101;
	public final static int UNKNOWNERROR = 104;
	public final static String BASEURL = "http://api.dms.aodianyun.com";
	protected String accessId;	//接口调用需要的参数
	protected String accessKey;	//接口调用需要的参数
	protected HttpHelper http;	//发起http请求的工具类
	public TisService(String accessId, String accessKey) {
		this.accessId = accessId;
		this.accessKey = accessKey;
		http = new HttpHelper();
	}
	//生成签名的函数
	private String Sign(String data,String key) {
		try {
			byte[] bytesKey = key.getBytes("UTF-8");
			final SecretKeySpec secretKey = new SecretKeySpec(bytesKey, "HmacSHA1");
			Mac mac = Mac.getInstance("HmacSHA1");
			mac.init(secretKey);
			final byte[] macData = mac.doFinal(data.getBytes("UTF-8"));
			String signature = Base64.encode(macData);
			return accessId+":"+signature;
		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException(e);
		} catch (InvalidKeyException e) {
			throw new RuntimeException(e);
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
	}
	public static String md5(String plainText) {  
        try {  
            MessageDigest md = MessageDigest.getInstance("MD5");  
			md.update(plainText.getBytes("UTF-8"));
            byte b[] = md.digest();  
            StringBuffer buf = new StringBuffer("");  
            for (int i,offset = 0; offset < b.length; offset++) {  
                i = b[offset];  
                if (i < 0)  
                    i += 256;  
                if (i < 16)  
                    buf.append("0");  
                buf.append(Integer.toHexString(i));  
            }  
            //32位
            return buf.toString();
            // 16位
            //return buf.toString().substring(8, 24);  
        } catch (NoSuchAlgorithmException e) {  
            e.printStackTrace();  
        } catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
        return null;  
    }
	//获取请求头
	private Map<String,String> getHttpHeaders(String path,String method,String body) {
		String signStr = "";
		long expire = new Date().getTime()/1000 + 3600;
		if(!"".equals(body)) {
			signStr = method + "\n" + path + "\n" + expire +  "\n"+ md5(body) + "\n";
		} else {
			signStr = method + "\n" + path + "\n" + expire +  "\n";
		}
		Map<String,String> headers = new HashMap<String,String>();
		headers.put("Authorization", "tis " + Sign(signStr,accessKey));	//签名
		headers.put("AD-Expire", expire+"");
		headers.put("Content-Type", "application/json");
		return headers;
	}
	//返回错误提示，最终会捕获ApiException，返回JSON形式的字符串
	protected JSONObject errorResp(Object code,String msg) {
		throw new ApiException("Flag",code,"FlagString",msg);
	}
	//返回错误提示，最终会捕获ParamApiException，返回JSON形式的字符串
	protected JSONObject paramErrorResp(String msg) {
		throw new ApiException("Flag",PARAMERROR,"FlagString",msg);
	}
	//返回错误提示，最终会捕获ApiException，返回JSON形式的字符串
	protected JSONObject formatErrorResp(String msg) {
		throw new ApiException("Flag",FORMATERROR,"FlagString",msg);
	}
	//工具函数，解析字符串为JSONObject对象、或者JSONArray对象
	private Object parseJSON(String str) {
    	JSONTokener parser = new JSONTokener(str);
    	Object obj;
		try {
			obj = parser.nextValue();
		} catch (Exception e) {
			obj = formatErrorResp("json format error : " + str);
		}
    	return obj;
	}
	//方便发起http请求的工具函数
	private JSONObject httpRequest(String path,String method,String body) {
    	try {
    		String result;
    		//根据method 发起不同的请求
    		if("POST".equals(method)) {
    			result = http.httpPostRequest(BASEURL+path, body, getHttpHeaders(path,"POST",body));
    		} else if("DELETE".equals(method)) {
    			result = http.httpDeleteRequest(BASEURL+path, getHttpHeaders(path,"DELETE",""));
    		} else {
    			result = http.httpGetRequest(BASEURL+path, getHttpHeaders(path,"GET",""));
    		}
    		JSONObject resp;
    		//返回的Info 要么是NULL，要么是具体的JSON数据
    		if("".equals(result)) {
    			resp = new JSONObject();
    		} else {
    			try {
    				resp = (JSONObject) parseJSON(result);
    			} catch (ClassCastException e) {
    				resp = formatErrorResp("json format error : " + result);
    			}
    		}
    		return resp;
    	} catch(HttpHelper.HttpApiException e) {
    		Object code;
    		String error;
    		try {
    			//如果返回的http状态码不对，就返回JSON格式的错误提示
    			JSONObject resp = (JSONObject) parseJSON(e.getMessage());
    			code = resp.get("code");
    			error = resp.getString("error");
			} catch (Exception e1) {
				return formatErrorResp("response format error");
			}
    		return errorResp(code,error);
    	}
	}
	//发起Http请求的工具函数，提供下面的具体接口调用使用
	private JSONObject httpGetRequest(String path) {
		return httpRequest(path,"GET",null);
	}
	//发起Http请求的工具函数，提供下面的具体接口调用使用
	private JSONObject httpPostRequest(String path,String body) {
		return httpRequest(path,"POST",body);
	}
	//从JSONObject中获取参数用的函数，为了省去异常捕获
	protected String getStringParam(JSONObject param,String field,String defVal) {
		try {
			return param.getString(field);
		} catch (JSONException e) {
			return defVal;
		}
	}
	//从JSONObject中获取参数用的函数，为了省去异常捕获
	protected String getStringParam(JSONObject param,String field) {
		try {
			return param.getString(field);
		} catch (JSONException e) {
			this.paramErrorResp("param "+field+" error");
		}
		return "";
	}
	protected String getEncodedStringParam(JSONObject param,String field) {
		try {
			return URLEncoder.encode(getStringParam(param,field), "UTF-8");
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
	}
	//从JSONObject中获取参数用的函数，为了省去异常捕获
	protected int getIntParam(JSONObject param,String field) {
		try {
			return param.getInt(field);
		} catch (JSONException e) {
			try {
				return Integer.parseInt(param.getString(field));
			} catch(Exception ex) {
				this.paramErrorResp("param "+field+" error");
			}
		}
		return 0;
	}
	//从JSONObject中获取参数用的函数，为了省去异常捕获
	protected int getIntParam(JSONObject param,String field,int defVal) {
		try {
			return param.getInt(field);
		} catch (JSONException e) {
			try {
				return Integer.parseInt(param.getString(field));
			} catch(Exception ex) {
				return defVal;
			}
		}
	}
	//将数据放入JSONObject对象中，为了方便，省去异常捕获
	public void putObjectParam(JSONObject param,String field,Object value) {
		try {
			param.put(field, value);
		} catch (JSONException e) {
			this.formatErrorResp("put "+value+" to json object error");
		}
	}
	//open api
	public JSONObject history(JSONObject params) {
		String tisId = getEncodedStringParam(params, "tisId");
		int skip = getIntParam(params, "skip", 0);
		int num = getIntParam(params, "num", 100);
		return httpGetRequest("/v1/tis/words/" + tisId + "/history?skip=" + skip + "&num=" + num);
	}
	public JSONObject sendMsg(JSONObject params) {
		String tisId = getEncodedStringParam(params, "tisId");
		String content = getStringParam(params, "content");
		JSONObject body = new JSONObject();
		putObjectParam(body,"content",content);
		return httpPostRequest("/v1/tis/words/" + tisId + "/auto",body.toString());
	}
	public JSONObject getFacesByGroup(JSONObject params) {
		String groupId = getEncodedStringParam(params, "groupId");
		int skip = getIntParam(params, "skip", 0);
		int num = getIntParam(params, "num", 100);
		return httpGetRequest("/v1/tis/faces?groupId=" + groupId + "&skip=" + skip + "&num=" + num);
	}
	public JSONObject getFaceGroups(JSONObject params) {
		String tisId = getEncodedStringParam(params, "tisId");
		int skip = getIntParam(params, "skip", 0);
		int num = getIntParam(params, "num", 100);
		return httpGetRequest("/v1/tis/faceGroups?instId=" + tisId + "&skip=" + skip + "&num=" + num);
	}
	public JSONObject getTisInst(JSONObject params) {
		String tisId = getEncodedStringParam(params, "tisId");
		return httpGetRequest("/v1/tis/inst/" + tisId);
	}
	public JSONObject getPackage(JSONObject params) {
		String pkgId = getEncodedStringParam(params, "pkgId");
		return httpGetRequest("/v1/tis/packages/" + pkgId);
	}
	/*
	public static void main(String[] args) {
		JSONObject obj = new JSONObject();
		new TisService("", "").getIntParam(obj, "xxx");
		System.out.println("...");
	}*/
}
