package com.aodianyun.web.api;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Map;
import java.util.Map.Entry;
import javax.servlet.http.HttpServletRequest;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import com.aodianyun.service.api.ApiException;
import com.aodianyun.service.api.TisService;

@RequestMapping("/api")
public class ApiController {
	
	// Flag 和 FlasString 是大部分接口中规定的字段，Flag代表错误代码，FlagString代表错误提示消息
	protected String errorField = "Flag";
	protected String msgField  = "FlagString";
	protected Class<?> apiParamType = JSONObject.class;
	protected ApiController() {}
	protected ApiController(String errorField,String msgField,Class<?> apiParamType) {
		this.errorField = errorField;
		this.msgField = msgField;
		this.apiParamType = apiParamType;
	}
	
	//方便返回 错误提示的工具函数
	protected JSONObject getErrorResponse(String errorField,Object errorCode,String msgField,String msg) {
		JSONObject obj = new JSONObject();
		try {
			obj.put(errorField, errorCode);
			obj.put(msgField, msg);
		} catch (JSONException e) {
			e.printStackTrace();
		}
		return obj;
	}
	//异常处理
	@ResponseBody
	@ExceptionHandler(Exception.class)
	public String handleException(HttpServletRequest request, Exception e) {
		return getErrorResponse(errorField,TisService.UNKNOWNERROR,msgField,e.getMessage()).toString();
	}
	@ResponseBody
	@ExceptionHandler(ApiException.class)
	public String handleApiException(HttpServletRequest request, ApiException e) {
		if(e.getData() != null)
			return e.getData().toString();
		return "";
	}
	//从resuest中读取参数，封装成JSONObject
	protected Object getParamsObject(HttpServletRequest request) throws JSONException 
	{
		Map<String, String[]> params = request.getParameterMap();
		JSONObject jsonObject = new JSONObject();
		//把参数封装成json对象
		for(Entry<String, String[]> item : params.entrySet()) {
			String[] values = item.getValue();
			if(values != null && values.length >0) {
				if(!item.getKey().endsWith("[]") && values.length == 1) {
					jsonObject.put(item.getKey(), values[0]);
				} else {
					//如果是  key[]=value1  或者 是  key=value1&key=value2... 形式的就封装成数组
					JSONArray array = new JSONArray();
					for(String value : values) {
						array.put(value);
					}
					jsonObject.put(item.getKey(), array);
				}
			}
		}
		return jsonObject;
	}
    public String execute(Object api,String method, HttpServletRequest request) throws Exception {
		Object params;
		try {
			params = getParamsObject(request);
		} catch(JSONException e) {
			return getErrorResponse(errorField, TisService.FORMATERROR, msgField, "param format error").toString();
		}
    	Method  m;
		try {
			m = api.getClass().getMethod(method, apiParamType);
		} catch (Exception e) {
			return getErrorResponse(errorField, TisService.UNKNOWNERROR, msgField, "method " + method + " error").toString();
		}
		try {
			return m.invoke(api, params).toString();	//将接口返回的JSONObject对象转成字符串形式的数据
		} catch (IllegalArgumentException e) {
			return getErrorResponse(errorField, TisService.UNKNOWNERROR, msgField, "method " + method + " IllegalArgumentException").toString();
		} catch (IllegalAccessException errorField) {
			return getErrorResponse("Flag", TisService.UNKNOWNERROR, msgField, "method " + method + " IllegalAccessException").toString();
		} catch (InvocationTargetException e) {
			throw (Exception)e.getCause();
		}
    }
}
